-- Used to create the SQLite file cities.sdb

-- to create the SQLite file cities.sdb invoke command
--   sqlite3 cities.sdb < cities.sdb

create table cities(name varchar(24), lat float, lon lat float);
insert into cities values('Amsterdam',52.4,4.9);
insert into cities values('Bonn',50.7,7.1);
insert into cities values('Rome',41.9,12.5);
