<?xml version='1.0' encoding='utf-8'?>

<!-- Very simple Jinja2 template -->
<cities>
    {% for city in cities %}
    <city>
        <name>{{ city.name }}</name>
        <lat>{{ city.lat }}</lat>
        <lon>{{ city.lon }}</lon>
    </city>
    {% endfor %}
</cities>
